/*****************************************************
Project : Controller PAM 6CH
Version : 1
Date    : 02.08.2013
Author  : Ilya Petrukhun
Company : IP
Comments: 


Chip type               : ATtiny2313A
AVR Core Clock frequency: 8,000000 MHz
Memory model            : Tiny
External RAM size       : 0
Data Stack size         : 32
*****************************************************/

#include <tiny2313a.h>
#include "Controller.h"

void interpretator (void);
void PWMfunc (void);
void MakeArrayPorts (void);
void initperif (void);
void initperem (void);


extern flash unsigned char Rules [];
register flash unsigned char * AdrRules = &Rules [0];

unsigned int			ChannelMask; 			//  
flash const unsigned char			TimeArray[14] = {
0, 0, 1, 3, 7, 15, 31, 31, 31, 15, 15, 31, 31, 31 
};			//    
flash const unsigned char			NumBits[14] = {
0, 1, 2, 3, 4, 6, 8, 7, 8, 5, 6, 8, 7, 8
};			//     

volatile unsigned char	CHlevel[NumChannel];	//  
unsigned char			BAM_Level_PortA[9];	//   BAM      
unsigned char			BAM_Level_PortB[9];
unsigned char			BAM_Level_PortD[9];
 
flash const unsigned char		Mask_Levels[10] =		//    
{
0xFF, 0x01, 0x02, 0x04, 0x08, 0x10, 0x20, 0x40, 0x80, 0x00
};
flash const unsigned char		BitPorts[NumChannel] =	//      
{
0x01, 0x02, 0x02, 0x01, 0x04, 0x08
};
const unsigned char 		*pPorts[NumChannel] =	//     
{
BAM_Level_PortD, 
BAM_Level_PortD, 
BAM_Level_PortA, 
BAM_Level_PortA,
BAM_Level_PortD, 
BAM_Level_PortD
};

register unsigned char		Command;				//   
register unsigned char		Number;					//    
unsigned char				CountCycle[MAX_NUMBER_CYCLES];	//  
flash unsigned char			*AddressCycle[MAX_NUMBER_CYCLES];
register unsigned char 		i;
register unsigned char 		j;
unsigned char		n;
unsigned char		m;



eeprom unsigned int Speed;  //      1/50 
register unsigned char	CountTick;             //   
unsigned int	CHset;	//    (1-8)
//register unsigned char	CHset1;	//    (9-16)
unsigned int	CHmask;	//     (1-8)
//register unsigned char	CHmask1;	//     (9-16)
unsigned char	CHspeed;	//    
bit						fTick;
bit						Flag;


void main(void)
{    
    initperif ();

    initperem ();


// Global enable interrupts
#asm("sei")

	while (1)
	{
 		if (fTick)
		{
			fTick = 0;
    	    interpretator (); 
    	    PWMfunc ();
		}
    	if ((n == 12) && Flag)	//         
		{
           	Flag = 0;
            MakeArrayPorts ();
        }
	}
}


void interpretator (void)
{
//   
	CountTick-- ;
    if (CountTick == 0) 	//     
    {                       //           
start:
      	if (*AdrRules == 0)	//   
       	{
        	AdrRules++;
        	Command = *AdrRules++;
            switch (Command)
            {  
            case START_CYCLE:
                Number = *AdrRules++;				//  
                CountCycle[Number] = *AdrRules++;	//    - 1
                AddressCycle[Number] = AdrRules;	//     
            	break;
                
            case END_CYCLE:
                Number = *AdrRules++;
                if (CountCycle[Number])				//     
                {
                	CountCycle[Number]--;			//  
                    AdrRules = AddressCycle[Number];	//   
                }
                else								//    
                {
                	AdrRules++;					//    
                }
            	break; 
            
            case FIRST:				//  
            default:
            	AdrRules = Rules;	            
            	break;
            }
            goto start;			//    ,     
        }
        else
        {
        	CountTick = *AdrRules++;//   
			CHset = *AdrRules++;	//   
			CHmask = *AdrRules++;	//   
			CHspeed = *AdrRules++;	//     
        } 
    }
}

void PWMfunc (void)
{
    //      
    for (i=0, ChannelMask = CHM0; i < NumChannel; i++)
    {
    	if ((CHmask & ChannelMask) == 0)	// 
        {
        	if ((CHset & ChannelMask) == 0)
            {
            	CHlevel[i] = 0;
            }
            else
            {
            	CHlevel[i] = 255;
            }
        }
        else 	//  
        {
        	if ((CHset & ChannelMask) == 0)		//  
            {
            	if (CHlevel[i] > CHspeed)
                {
                	CHlevel[i] -= CHspeed;
                }
                else
                {
                	CHlevel[i] = 0;
                }
	        }
            else	//  	
            {
            
            	if ((255 - CHlevel[i]) > CHspeed)	
                {
                	CHlevel[i] += CHspeed;
                }
                else
                {
                	CHlevel[i] = 255; 	//     
                }
            }
        }
    ChannelMask <<= 1;
    }
    #asm
    wdr
	#endasm  
    Flag = 1; 
}

void MakeArrayPorts (void)
{
	register unsigned char		*pPort;
    
	for (i = 0; i < 9; i++)
    {        
    	//0x03;						//    0  1 -   
    	BAM_Level_PortA[i] = BAM_Level_PortD[i] = BAM_Level_PortB[i] = 0; 	//   
        for (j = 0; j < NumChannel; j++)
        {
        	pPort = pPorts[j] + i;
        	if (i == 0)
            {
            	if (CHlevel[j] == Mask_Levels[i])	//       
                {
                	*pPort |= BitPorts[j];
                }
            }
    		else if (CHlevel[j] & Mask_Levels[i])	//   
            {  
            	*pPort |= BitPorts[j]; 
            }
        } 
    }
}


void initperem (void)
{

	for (i = 0; i < MAX_NUMBER_CYCLES; i++)
		CountCycle[i] = 0;	//   
    for (i = 0; i < NumChannel; i++) 
    	CHlevel[i] = 0;   
    for (i = 0; i < 9; i++)
    {
    	BAM_Level_PortA[i] = BAM_Level_PortB[i] = BAM_Level_PortD[i] = 0;
        //0x03; 
    }   

	fTick = 0;
    Flag = 0;
	AdrRules = Rules;	//   
    CountTick = 1;
}

